//
//  TableViewController.swift
//  TestBackendlessdataCollection
//
//  Created by Patrick Delaney on 19/05/2020.
//  Copyright © 2020 Patrick Delaney. All rights reserved.
//

import UIKit
import Backendless

@objcMembers class testTable: NSObject, BLIdentifiable {
    var objectId: String?
    
    var testColumn : String?
}


class TableViewController: UITableViewController {

    var items: BackendlessDataCollection?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        initBackendlessData()
    }

    // MARK: - Table view data source
    override func numberOfSections(in tableView: UITableView) -> Int { return 1}
    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int { return items?.count ?? 0 }
    override func tableView(_ tableView: UITableView, canEditRowAt indexPath: IndexPath) -> Bool { return true }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "testCell", for: indexPath)

        let item = items![indexPath.row] as! testTable
        cell.textLabel?.text = item.testColumn
        
        return cell
    }

    override func tableView(_ tableView: UITableView, commit editingStyle: UITableViewCell.EditingStyle, forRowAt indexPath: IndexPath) {
        if editingStyle == .delete {
            print ("BEFORE REMOVE COUNT - \(items?.count)")
            let _ = items?.remove(at: indexPath.row)
            print ("AFTER REMOVE COUNT - \(items?.count)")
            
            tableView.deleteRows(at: [indexPath], with: .fade)
        }
    }

    //MARK: - Functions
    func initBackendlessData() {
        // Load InventoryItems From Backendless
        items = nil // ensure we're dealing with an empty array
        let queryBuilder = DataQueryBuilder()
        queryBuilder.setPageSize(pageSize: 100)
        
        items = BackendlessDataCollection(entityType: testTable.self, queryBuilder: queryBuilder)
    }
}
